# -*- coding: utf-8 -*-
from helpers import use_cassette

from test_backend import *
from test_forms import *
from test_models import *
from test_client_views import *
from test_sso_client import *
from test_sso_views import *
from test_p3p_middleware import *
from test_client_api_methods import *
from test_decorators import *
